from rest_framework import serializers

from solid_backend.photograph.serializers import PhotographSerializer

from .models import Message


class MessageSerializer(serializers.ModelSerializer):
    img = PhotographSerializer()

    class Meta:
        model = Message
        fields = "__all__"



from django.contrib import admin

from .forms import SlideshowAdminForm, SlideshowInlineFormSet
from .models import Slideshow, SlideshowImage, SlideshowPage


class SlideshowPageInline(admin.TabularInline):
    model = SlideshowPage
    formset = SlideshowInlineFormSet
    extra = 1


class SlideshowImageInline(admin.TabularInline):
    model = SlideshowImage
    formset = SlideshowInlineFormSet
    extra = 1


class SlideshowAdmin(admin.ModelAdmin):
    form = SlideshowAdminForm
    fields = [("position", "active"), "title", "img"]
    list_display = ["id", "title", "position", "img", "active"]
    list_display_links = ["title"]
    inlines = [SlideshowPageInline]


admin.site.register(Slideshow, SlideshowAdmin)


class SlideshowPageAdmin(admin.ModelAdmin):
    form = SlideshowAdminForm
    list_display = ["id", "title", "position", "show"]
    list_display_links = ["title"]
    inlines = [SlideshowImageInline]


admin.site.register(SlideshowPage, SlideshowPageAdmin)


class SlideshowImageAdmin(admin.ModelAdmin):
    form = SlideshowAdminForm
    list_display = ["id", "title", "position", "show_page", "img"]
    list_display_links = ["title"]

    def show_page(self, obj):
        return "{} : {}".format(
            getattr(SlideshowPage.objects.get(id=obj.page.id), "show"), obj.page
        )

    show_page.short_description = "Show : Page"


admin.site.register(SlideshowImage, SlideshowImageAdmin)



# Generated by Django 3.0.9 on 2020-11-03 10:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("photograph", "0005_implement_deep_zoom_fields"),
    ]

    operations = [
        migrations.AddField(
            model_name="photograph",
            name="profile_position",
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.0.7 on 2020-07-21 14:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("message", "0002_correct_typo"),
    ]

    operations = [
        migrations.AlterField(
            model_name="message",
            name="valid_to",
            field=models.DateField(blank=True, null=True),
        ),
    ]



#!/usr/bin/env python3

#
#  Deep Zoom Tools
#
#  Copyright (c) 2008-2019, Daniel Gasienica <daniel@gasienica.ch>
#  Copyright (c) 2008-2011, OpenZoom <http://openzoom.org>
#  Copyright (c) 2010, Boris Bluntschli <boris@bluntschli.ch>
#  Copyright (c) 2008, Kapil Thangavelu <kapil.foss@gmail.com>
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without modification,
#  are permitted provided that the following conditions are met:
#
#      1. Redistributions of source code must retain the above copyright notice,
#         this list of conditions and the following disclaimer.
#
#      2. Redistributions in binary form must reproduce the above copyright
#         notice, this list of conditions and the following disclaimer in the
#         documentation and/or other materials provided with the distribution.
#
#      3. Neither the name of OpenZoom nor the names of its contributors may be used
#         to endorse or promote products derived from this software without
#         specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
#  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
#  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import io
import math
import optparse
import os
import shutil
from urllib.parse import urlparse
import sys
import time
import urllib.request
import warnings
import xml.dom.minidom

import PIL.Image

from collections import deque


NS_DEEPZOOM = "http://schemas.microsoft.com/deepzoom/2008"

DEFAULT_RESIZE_FILTER = PIL.Image.ANTIALIAS
DEFAULT_IMAGE_FORMAT = "jpg"

RESIZE_FILTERS = {
    "cubic": PIL.Image.CUBIC,
    "bilinear": PIL.Image.BILINEAR,
    "bicubic": PIL.Image.BICUBIC,
    "nearest": PIL.Image.NEAREST,
    "antialias": PIL.Image.ANTIALIAS,
}

IMAGE_FORMATS = {
    "jpg": "jpg",
    "png": "png",
}


class DeepZoomImageDescriptor(object):
    def __init__(
        self, width=None, height=None, tile_size=254, tile_overlap=1, tile_format="jpg"
    ):
        self.width = width
        self.height = height
        self.tile_size = tile_size
        self.tile_overlap = tile_overlap
        self.tile_format = tile_format
        self._num_levels = None

    def open(self, source):
        """Intialize descriptor from an existing descriptor file."""
        doc = xml.dom.minidom.parse(safe_open(source))
        image = doc.getElementsByTagName("Image")[0]
        size = doc.getElementsByTagName("Size")[0]
        self.width = int(size.getAttribute("Width"))
        self.height = int(size.getAttribute("Height"))
        self.tile_size = int(image.getAttribute("TileSize"))
        self.tile_overlap = int(image.getAttribute("Overlap"))
        self.tile_format = image.getAttribute("Format")

    def save(self, destination):
        """Save descriptor file."""
        file = open(destination, "wb")
        doc = xml.dom.minidom.Document()
        image = doc.createElementNS(NS_DEEPZOOM, "Image")
        image.setAttribute("xmlns", NS_DEEPZOOM)
        image.setAttribute("TileSize", str(self.tile_size))
        image.setAttribute("Overlap", str(self.tile_overlap))
        image.setAttribute("Format", str(self.tile_format))
        size = doc.createElementNS(NS_DEEPZOOM, "Size")
        size.setAttribute("Width", str(self.width))
        size.setAttribute("Height", str(self.height))
        image.appendChild(size)
        doc.appendChild(image)
        descriptor = doc.toxml(encoding="UTF-8")
        file.write(descriptor)
        file.close()

    @classmethod
    def remove(self, filename):
        """Remove descriptor file (DZI) and tiles folder."""
        _remove(filename)

    @property
    def num_levels(self):
        """Number of levels in the pyramid."""
        if self._num_levels is None:
            max_dimension = max(self.width, self.height)
            self._num_levels = int(math.ceil(math.log(max_dimension, 2))) + 1
        return self._num_levels

    def get_scale(self, level):
        """Scale of a pyramid level."""
        assert 0 <= level and level < self.num_levels, "Invalid pyramid level"
        max_level = self.num_levels - 1
        return math.pow(0.5, max_level - level)

    def get_dimensions(self, level):
        """Dimensions of level (width, height)"""
        assert 0 <= level and level < self.num_levels, "Invalid pyramid level"
        scale = self.get_scale(level)
        width = int(math.ceil(self.width * scale))
        height = int(math.ceil(self.height * scale))
        return (width, height)

    def get_num_tiles(self, level):
        """Number of tiles (columns, rows)"""
        assert 0 <= level and level < self.num_levels, "Invalid pyramid level"
        w, h = self.get_dimensions(level)
        return (
            int(math.ceil(float(w) / self.tile_size)),
            int(math.ceil(float(h) / self.tile_size)),
        )

    def get_tile_bounds(self, level, column, row):
        """Bounding box of the tile (x1, y1, x2, y2)"""
        assert 0 <= level and level < self.num_levels, "Invalid pyramid level"
        offset_x = 0 if column == 0 else self.tile_overlap
        offset_y = 0 if row == 0 else self.tile_overlap
        x = (column * self.tile_size) - offset_x
        y = (row * self.tile_size) - offset_y
        level_width, level_height = self.get_dimensions(level)
        w = self.tile_size + (1 if column == 0 else 2) * self.tile_overlap
        h = self.tile_size + (1 if row == 0 else 2) * self.tile_overlap
        w = min(w, level_width - x)
        h = min(h, level_height - y)
        return (x, y, x + w, y + h)


class DeepZoomCollection(object):
    def __init__(
        self,
        filename,
        image_quality=0.8,
        max_level=7,
        tile_size=256,
        tile_format="jpg",
        items=[],
    ):
        self.source = filename
        self.image_quality = image_quality
        self.tile_size = tile_size
        self.max_level = max_level
        self.tile_format = tile_format
        self.items = deque(items)
        self.next_item_id = len(self.items)
        # XML
        self.doc = xml.dom.minidom.Document()
        collection = self.doc.createElementNS(NS_DEEPZOOM, "Collection")
        collection.setAttribute("xmlns", NS_DEEPZOOM)
        collection.setAttribute("MaxLevel", str(self.max_level))
        collection.setAttribute("TileSize", str(self.tile_size))
        collection.setAttribute("Format", str(self.tile_format))
        collection.setAttribute("Quality", str(self.image_quality))
        # TODO: Append items passed in as argument
        items = self.doc.createElementNS(NS_DEEPZOOM, "Items")
        collection.appendChild(items)
        collection.setAttribute("NextItemId", str(self.next_item_id))
        self.doc.appendChild(collection)

    @classmethod
    def from_file(self, filename):
        """Open collection descriptor."""
        doc = xml.dom.minidom.parse(safe_open(filename))
        collection = doc.getElementsByTagName("Collection")[0]
        image_quality = float(collection.getAttribute("Quality"))
        max_level = int(collection.getAttribute("MaxLevel"))
        tile_size = int(collection.getAttribute("TileSize"))
        tile_format = collection.getAttribute("Format")
        items = [
            DeepZoomCollectionItem.from_xml(item)
            for item in doc.getElementsByTagName("I")
        ]

        collection = DeepZoomCollection(
            filename,
            image_quality=image_quality,
            max_level=max_level,
            tile_size=tile_size,
            tile_format=tile_format,
            items=items,
        )
        return collection

    @classmethod
    def remove(self, filename):
        """Remove collection file (DZC) and tiles folder."""
        _remove(filename)

    def append(self, source):
        descriptor = DeepZoomImageDescriptor()
        descriptor.open(source)
        item = DeepZoomCollectionItem(
            source, descriptor.width, descriptor.height, id=self.next_item_id
        )
        self.items.append(item)
        self.next_item_id += 1

    def save(self, pretty_print_xml=False):
        """Save collection descriptor."""
        collection = self.doc.getElementsByTagName("Collection")[0]
        items = self.doc.getElementsByTagName("Items")[0]
        while len(self.items) > 0:
            item = self.items.popleft()
            i = self.doc.createElementNS(NS_DEEPZOOM, "I")
            i.setAttribute("Id", str(item.id))
            i.setAttribute("N", str(item.id))
            i.setAttribute("Source", item.source)
            # Size
            size = self.doc.createElementNS(NS_DEEPZOOM, "Size")
            size.setAttribute("Width", str(item.width))
            size.setAttribute("Height", str(item.height))
            i.appendChild(size)
            items.appendChild(i)
            self._append_image(item.source, item.id)
        collection.setAttribute("NextItemId", str(self.next_item_id))
        with open(self.source, "wb") as f:
            if pretty_print_xml:
                xml = self.doc.toprettyxml(encoding="UTF-8")
            else:
                xml = self.doc.toxml(encoding="UTF-8")
            f.write(xml)

    def _append_image(self, path, i):
        descriptor = DeepZoomImageDescriptor()
        descriptor.open(path)
        files_path = _get_or_create_path(_get_files_path(self.source))
        for level in reversed(range(self.max_level + 1)):
            level_path = _get_or_create_path("%s/%s" % (files_path, level))
            level_size = 2 ** level
            images_per_tile = int(math.floor(self.tile_size / level_size))
            column, row = self.get_tile_position(i, level, self.tile_size)
            tile_path = "%s/%s_%s.%s" % (level_path, column, row, self.tile_format)
            if not os.path.exists(tile_path):
                tile_image = PIL.Image.new("RGB", (self.tile_size, self.tile_size))
                q = int(self.image_quality * 100)
                tile_image.save(tile_path, "JPEG", quality=q)
            tile_image = PIL.Image.open(tile_path)
            source_path = "%s/%s/%s_%s.%s" % (
                _get_files_path(path),
                level,
                0,
                0,
                descriptor.tile_format,
            )
            # Local
            if os.path.exists(source_path):
                try:
                    source_image = PIL.Image.open(safe_open(source_path))
                except IOError:
                    warnings.warn("Skipped invalid level: %s" % source_path)
                    continue
            # Remote
            else:
                if level == self.max_level:
                    try:
                        source_image = PIL.Image.open(safe_open(source_path))
                    except IOError:
                        warnings.warn("Skipped invalid image: %s" % source_path)
                        return
                    # Expected width & height of the tile
                    e_w, e_h = descriptor.get_dimensions(level)
                    # Actual width & height of the tile
                    w, h = source_image.size
                    # Correct tile because of IIP bug where low-level tiles have
                    # wrong dimensions (they are too large)
                    if w != e_w or h != e_h:
                        # Resize incorrect tile to correct size
                        source_image = source_image.resize(
                            (e_w, e_h), PIL.Image.ANTIALIAS
                        )
                        # Store new dimensions
                        w, h = e_w, e_h
                else:
                    w = int(math.ceil(w * 0.5))
                    h = int(math.ceil(h * 0.5))
                    source_image.thumbnail((w, h), PIL.Image.ANTIALIAS)
            column, row = self.get_position(i)
            x = (column % images_per_tile) * level_size
            y = (row % images_per_tile) * level_size
            tile_image.paste(source_image, (x, y))
            tile_image.save(tile_path)

    def get_position(self, z_order):
        """Returns position (column, row) from given Z-order (Morton number.)"""
        column = 0
        row = 0
        for i in range(0, 32, 2):
            offset = i // 2
            # column
            column_offset = i
            column_mask = 1 << column_offset
            column_value = (z_order & column_mask) >> column_offset
            column |= column_value << offset
            # row
            row_offset = i + 1
            row_mask = 1 << row_offset
            row_value = (z_order & row_mask) >> row_offset
            row |= row_value << offset
        return int(column), int(row)

    def get_z_order(self, column, row):
        """Returns the Z-order (Morton number) from given position."""
        z_order = 0
        for i in range(32):
            z_order |= (column & 1 << i) << i | (row & 1 << i) << (i + 1)
        return z_order

    def get_tile_position(self, z_order, level, tile_size):
        level_size = 2 ** level
        x, y = self.get_position(z_order)
        return (
            int(math.floor((x * level_size) / tile_size)),
            int(math.floor((y * level_size) / tile_size)),
        )


class DeepZoomCollectionItem(object):
    def __init__(self, source, width, height, id=0):
        self.id = id
        self.source = source
        self.width = width
        self.height = height

    @classmethod
    def from_xml(cls, xml):
        id = int(xml.getAttribute("Id"))
        source = xml.getAttribute("Source")
        size = xml.getElementsByTagName("Size")[0]
        width = int(size.getAttribute("Width"))
        height = int(size.getAttribute("Height"))
        return DeepZoomCollectionItem(source, width, height, id)


class ImageCreator(object):
    """Creates Deep Zoom images."""

    def __init__(
        self,
        tile_size=254,
        tile_overlap=1,
        tile_format="jpg",
        image_quality=0.8,
        resize_filter=None,
        copy_metadata=False,
    ):
        self.tile_size = int(tile_size)
        self.tile_format = tile_format
        self.tile_overlap = _clamp(int(tile_overlap), 0, 10)
        self.image_quality = _clamp(image_quality, 0, 1.0)
        if not tile_format in IMAGE_FORMATS:
            self.tile_format = DEFAULT_IMAGE_FORMAT
        self.resize_filter = resize_filter
        self.copy_metadata = copy_metadata

    def get_image(self, level):
        """Returns the bitmap image at the given level."""
        assert (
            0 <= level and level < self.descriptor.num_levels
        ), "Invalid pyramid level"
        width, height = self.descriptor.get_dimensions(level)
        # don't transform to what we already have
        if self.descriptor.width == width and self.descriptor.height == height:
            return self.image
        if (self.resize_filter is None) or (self.resize_filter not in RESIZE_FILTERS):
            return self.image.resize((width, height), PIL.Image.ANTIALIAS)
        return self.image.resize((width, height), RESIZE_FILTERS[self.resize_filter])

    def tiles(self, level):
        """Iterator for all tiles in the given level. Returns (column, row) of a tile."""
        columns, rows = self.descriptor.get_num_tiles(level)
        for column in range(columns):
            for row in range(rows):
                yield (column, row)

    def create(self, source, destination):
        """Creates Deep Zoom image from source file and saves it to destination."""
        self.image = PIL.Image.open(safe_open(source))
        width, height = self.image.size
        self.descriptor = DeepZoomImageDescriptor(
            width=width,
            height=height,
            tile_size=self.tile_size,
            tile_overlap=self.tile_overlap,
            tile_format=self.tile_format,
        )
        # Create tiles
        image_files = _get_or_create_path(_get_files_path(destination))
        for level in range(self.descriptor.num_levels):
            level_dir = _get_or_create_path(os.path.join(image_files, str(level)))
            level_image = self.get_image(level)
            for (column, row) in self.tiles(level):
                bounds = self.descriptor.get_tile_bounds(level, column, row)
                tile = level_image.crop(bounds)
                format = self.descriptor.tile_format
                tile_path = os.path.join(level_dir, "%s_%s.%s" % (column, row, format))
                tile_file = open(tile_path, "wb")
                if self.descriptor.tile_format == "jpg":
                    jpeg_quality = int(self.image_quality * 100)
                    tile.save(tile_file, "JPEG", quality=jpeg_quality)
                else:
                    tile.save(tile_file)
        # Create descriptor
        self.descriptor.save(destination)


class CollectionCreator(object):
    """Creates Deep Zoom collections."""

    def __init__(
        self,
        image_quality=0.8,
        tile_size=256,
        max_level=7,
        tile_format="jpg",
        copy_metadata=False,
    ):
        self.image_quality = image_quality
        self.tile_size = tile_size
        self.max_level = max_level
        self.tile_format = tile_format
        # TODO
        self.copy_metadata = copy_metadata

    def create(self, images, destination):
        """Creates a Deep Zoom collection from a list of images."""
        collection = DeepZoomCollection(
            destination,
            image_quality=self.image_quality,
            max_level=self.max_level,
            tile_size=self.tile_size,
            tile_format=self.tile_format,
        )
        for image in images:
            collection.append(image)
        collection.save()


################################################################################


def retry(attempts, backoff=2):
    """Retries a function or method until it returns or
    the number of attempts has been reached."""

    if backoff <= 1:
        raise ValueError("backoff must be greater than 1")

    attempts = int(math.floor(attempts))
    if attempts < 0:
        raise ValueError("attempts must be 0 or greater")

    def deco_retry(f):
        def f_retry(*args, **kwargs):
            last_exception = None
            for _ in range(attempts):
                try:
                    return f(*args, **kwargs)
                except Exception as exception:
                    last_exception = exception
                    time.sleep(backoff ** (attempts + 1))
            raise last_exception

        return f_retry

    return deco_retry


def _get_or_create_path(path):
    if not os.path.exists(path):
        os.makedirs(path)
    return path


def _clamp(val, min, max):
    if val < min:
        return min
    elif val > max:
        return max
    return val


def _get_files_path(path):
    return os.path.splitext(path)[0] + "_files"


def _remove(path):
    os.remove(path)
    tiles_path = _get_files_path(path)
    shutil.rmtree(tiles_path)


@retry(3)
def safe_open(path):
    # `urllib` in Python 2 supported both local paths as well as URLs. To
    # continue this in Python 3, we manually add `file://` prefix if `path` is
    # not a URL. This change is isolated to this function as we want the output
    # XML to still have the original input paths instead of absolute paths:
    has_scheme = bool(urlparse(path).scheme)
    normalized_path = f"file://{os.path.abspath(path)}" if not has_scheme else path
    return io.BytesIO(urllib.request.urlopen(normalized_path).read())


################################################################################


def main():
    parser = optparse.OptionParser(usage="Usage: %prog [options] filename")

    parser.add_option(
        "-d",
        "--destination",
        dest="destination",
        help="Set the destination of the output.",
    )
    parser.add_option(
        "-s",
        "--tile_size",
        dest="tile_size",
        type="int",
        default=254,
        help="Size of the tiles. Default: 254",
    )
    parser.add_option(
        "-f",
        "--tile_format",
        dest="tile_format",
        default=DEFAULT_IMAGE_FORMAT,
        help="Image format of the tiles (jpg or png). Default: jpg",
    )
    parser.add_option(
        "-o",
        "--tile_overlap",
        dest="tile_overlap",
        type="int",
        default=1,
        help="Overlap of the tiles in pixels (0-10). Default: 1",
    )
    parser.add_option(
        "-q",
        "--image_quality",
        dest="image_quality",
        type="float",
        default=0.8,
        help="Quality of the image output (0-1). Default: 0.8",
    )
    parser.add_option(
        "-r",
        "--resize_filter",
        dest="resize_filter",
        default=DEFAULT_RESIZE_FILTER,
        help="Type of filter for resizing (bicubic, nearest, bilinear, antialias (best). Default: antialias",
    )

    (options, args) = parser.parse_args()

    if not args:
        parser.print_help()
        sys.exit(1)

    source = args[0]

    if not options.destination:
        if os.path.exists(source):
            options.destination = os.path.splitext(source)[0] + ".dzi"
        else:
            options.destination = os.path.splitext(os.path.basename(source))[0] + ".dzi"
    if options.resize_filter and options.resize_filter in RESIZE_FILTERS:
        options.resize_filter = RESIZE_FILTERS[options.resize_filter]

    creator = ImageCreator(
        tile_size=options.tile_size,
        tile_format=options.tile_format,
        image_quality=options.image_quality,
        resize_filter=options.resize_filter,
    )
    creator.create(source, options.destination)


if __name__ == "__main__":
    main()



import pytest

from solid_backend.message.models import Message
from solid_backend.photograph.models import Photograph


@pytest.fixture
def message_model_class():
    return Message


@pytest.fixture
def photograph_model_class():
    return Photograph



import pytest

from solid_backend.photograph.models import DeepZoom, Photograph


@pytest.fixture
def deepzoom_model_class():
    return DeepZoom


@pytest.fixture
def photograph_model_class():
    return Photograph



from django import forms
from django.contrib import admin

from .models import Message


class DateOrderForm(forms.ModelForm):
    """
    Validate whether a validity start date and end date are in the correct order.
    """

    def clean(self):
        valid_from = self.cleaned_data.get("valid_from")
        valid_to = self.cleaned_data.get("valid_to")

        if valid_to is not None and valid_to < valid_from:
            error_message = "Ensure the end date is equal with or after the start date."
            self.add_error("valid_to", error_message)

        super(DateOrderForm, self).clean()


class MessageAdmin(admin.ModelAdmin):
    form = DateOrderForm
    list_display = ["id", "type", "title", "valid_from", "valid_to", "img"]


admin.site.register(Message, MessageAdmin)



# Generated by Django 3.0.9 on 2020-10-29 15:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("slideshow", "0008_add_active_and_position_field_slideshow_model"),
    ]

    operations = [
        migrations.AlterField(
            model_name="slideshow",
            name="position",
            field=models.PositiveSmallIntegerField(),
        ),
        migrations.AlterField(
            model_name="slideshowimage",
            name="position",
            field=models.PositiveSmallIntegerField(),
        ),
        migrations.AlterField(
            model_name="slideshowpage",
            name="position",
            field=models.PositiveSmallIntegerField(),
        ),
    ]



from django import forms

from .models import Slideshow, SlideshowImage, SlideshowPage


class SlideshowInlineFormSet(forms.BaseInlineFormSet):
    """
    Validate if position is available and order froms by position.
    """

    def clean(self):
        super().clean()

        positions = []

        for form in self.forms:
            if form.cleaned_data.get("DELETE"):
                continue
            position = form.cleaned_data.get("position")
            positions.append(position)

            if positions.count(position) > 1:
                form.add_error("position", "This position is not available.")

    def get_queryset(self):
        return super().get_queryset().order_by("position")


class SlideshowAdminForm(forms.ModelForm):
    """
    Validate if position is available.
    """

    def clean(self):
        super().clean()

        position_field = self.fields.get("position")
        initial_position = self.get_initial_for_field(position_field, "position")
        position = self.cleaned_data.get("position")
        model = type(self.instance)

        if model == Slideshow:
            objects = model.objects.all()
        elif model == SlideshowPage:
            objects = model.objects.filter(show=self.cleaned_data.get("show"))
        elif model == SlideshowImage:
            objects = model.objects.filter(page=self.cleaned_data.get("page"))
        else:
            raise TypeError("Only Slideshow, SlideshowPage, SlideshowImage allowed.")

        if position_field.has_changed(initial_position, position):
            for obj in objects:
                if position == obj.position:
                    self.add_error("position", "This position is not available.")



from django import forms
from django.contrib.contenttypes.forms import BaseGenericInlineFormSet
from mutagen import File

from .models import Photograph


class PhotographInlineFormSet(BaseGenericInlineFormSet):
    """
    Validate if profile position is available and order froms by profile position.
    """

    def clean(self):
        super().clean()

        positions = []

        for form in self.forms:
            if form.cleaned_data.get("DELETE"):
                continue
            position = form.cleaned_data.get("profile_position")
            positions.append(position)

            if positions.count(position) > 1:
                form.add_error("profile_position", "This position is not available.")

    def get_queryset(self):
        return super().get_queryset().order_by("profile_position")


class PhotographAdminForm(forms.ModelForm):
    """
    Calculate the scale and determine the audio duration.
    """

    LENGTH_UNIT_CHOICES = [
        (1.00, "m"),
        (0.01, "cm"),
        (0.001, "mm"),
    ]
    length_value = forms.FloatField(
        min_value=0.001,
        required=False,
        label="Scale calculator",
        help_text="Längenwert",
    )
    length_unit = forms.TypedChoiceField(
        coerce=float,
        choices=LENGTH_UNIT_CHOICES,
        initial=LENGTH_UNIT_CHOICES[1][0],
        label="",
        help_text="Längeneinheit",
    )
    pixel_number = forms.IntegerField(
        max_value=32767,
        min_value=0,
        required=False,
        label="",
        help_text='Pixelanzahl ("0" löscht den Wert von Scale)',
    )

    def save(self, commit=True):
        length_value = self.cleaned_data.get("length_value")
        length_unit = self.cleaned_data.get("length_unit")
        pixel_number = self.cleaned_data.get("pixel_number")

        instance = super(PhotographAdminForm, self).save(commit=False)

        if length_value and pixel_number:
            instance.img_original_scale = length_value * length_unit / pixel_number

        if pixel_number == 0:
            instance.img_original_scale = None

        if instance.audio:
            instance.audio_duration = File(instance.audio).info.length
        else:
            instance.audio_duration = None

        if commit:
            instance.save()

        return instance



from datetime import date
from os import makedirs, path
from shutil import rmtree

import django.db.models.options as options
from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from stdimage import JPEGField

from solid_backend.openzoom import deepzoom

options.DEFAULT_NAMES = options.DEFAULT_NAMES + ("image_field_name",)


class DeepZoom(models.Model):
    """
    Abstract Model for creation of Deep Zoom images
    """

    dzi_option = models.BooleanField(default=False, verbose_name="Deep Zoom option")
    dzi_file = models.FileField(
        null=True, editable=False, verbose_name="Deep Zoom file"
    )

    def create_deepzoom_files(self, upload_to="dzi"):
        # Generate Deep Zoom directory and file name form the image file name.
        image_absolute_path_file = getattr(self, self._meta.image_field_name).path
        slug = path.basename(image_absolute_path_file).split(".")[0]
        dzi_file_name = slug + ".dzi"
        dzi_absolute_path = path.join(settings.MEDIA_ROOT, upload_to, slug)
        dzi_absolute_path_file = path.join(dzi_absolute_path, dzi_file_name)
        dzi_relative_path_file = path.join(upload_to, slug, dzi_file_name)

        if self.dzi_file:
            if self.dzi_file.name == dzi_relative_path_file:
                return  # No update
            else:
                # Delete existing Deep Zoom directory before update.
                self.delete_deepzoom_files()

        # Create Deep Zoom image files.
        makedirs(dzi_absolute_path)
        creator = deepzoom.ImageCreator(
            tile_size=254,
            tile_overlap=1,
            tile_format="jpg",
            image_quality=0.9,
            resize_filter="antialias",
        )
        creator.create(image_absolute_path_file, dzi_absolute_path_file)
        self.dzi_file = dzi_relative_path_file

    def delete_deepzoom_files(self):
        rmtree(path.dirname(self.dzi_file.path))
        self.dzi_file = None

    def save(self, *args, **kwargs):
        if self.dzi_option:
            super().save(*args, **kwargs)  # Get image file path.
            self.create_deepzoom_files()
        elif self.dzi_file:
            self.delete_deepzoom_files()

        super().save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        if self.dzi_file:
            self.delete_deepzoom_files()
        super().delete(*args, **kwargs)

    class Meta:
        abstract = True
        # An option 'image_field_name' must be set in the child class.


class Photograph(DeepZoom):
    """
    Model for a photograph.
    """

    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, null=True)
    object_id = models.PositiveIntegerField(default=0)
    profile = GenericForeignKey("content_type", "object_id")
    profile_position = models.PositiveSmallIntegerField(null=True, blank=True)

    img = JPEGField(
        upload_to="photograph/",
        width_field="img_original_width",
        height_field="img_original_height",
        variations={
            "large": (1200, None),
            "medium": (900, None),
            "small": (600, None),
            "thumbnail": (100, 100, True),
        },
        db_index=True,
        delete_orphans=True,
    )
    img_original_width = models.PositiveSmallIntegerField(
        editable=False, verbose_name="img width"
    )
    img_original_height = models.PositiveSmallIntegerField(
        editable=False, verbose_name="img height"
    )
    img_original_scale = models.FloatField(verbose_name="scale", null=True, blank=True,)

    img_alt = models.CharField(max_length=200)
    description = models.TextField(
        default="", blank=True, verbose_name="description (Markdown)"
    )
    audio = models.FileField(upload_to="audio/", null=True, blank=True)
    audio_duration = models.FloatField(null=True, editable=False)

    date = models.DateField(
        null=True, blank=True, help_text="Datum der Lichtbildaufnahme"
    )
    author = models.CharField(max_length=100, default="", blank=True)
    license = models.CharField(max_length=100, default="", blank=True)

    def __str__(self):
        return str(self.img)

    class Meta:
        image_field_name = "img"



# Generated by Django 3.0.9 on 2020-08-24 11:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("photograph", "0004_add_verbose_names"),
    ]

    operations = [
        migrations.AddField(
            model_name="photograph",
            name="dzi_file",
            field=models.FileField(
                editable=False, null=True, upload_to="", verbose_name="Deep Zoom file"
            ),
        ),
        migrations.AddField(
            model_name="photograph",
            name="dzi_option",
            field=models.BooleanField(default=False, verbose_name="Deep Zoom option"),
        ),
    ]



# Generated by Django 3.0.9 on 2020-10-29 15:09

from django.db import migrations, models


def set_position(apps, schema_editor):
    """
    Set position value for Slideshow.
    """
    Slideshow = apps.get_model("slideshow", "Slideshow")

    position = 1

    for obj in Slideshow.objects.order_by("id"):
        obj.position = position
        obj.save()
        position += 1


class Migration(migrations.Migration):

    dependencies = [
        ("slideshow", "0007_assign_photograph_as_img_field_slideshow_image_model"),
    ]

    operations = [
        migrations.AddField(
            model_name="slideshow",
            name="active",
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name="slideshow",
            name="position",
            field=models.PositiveSmallIntegerField(default=1),
        ),
        migrations.RunPython(set_position, migrations.RunPython.noop),
    ]



# Generated by Django 3.0.7 on 2020-07-28 14:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("photograph", "0003_create_generic_foreign_key"),
    ]

    operations = [
        migrations.AlterField(
            model_name="photograph",
            name="img_original_height",
            field=models.PositiveSmallIntegerField(
                editable=False, verbose_name="img height"
            ),
        ),
        migrations.AlterField(
            model_name="photograph",
            name="img_original_width",
            field=models.PositiveSmallIntegerField(
                editable=False, verbose_name="img width"
            ),
        ),
    ]



from django.conf import settings
from django.conf.urls.static import static
from django.urls import include, path

urlpatterns = [
    path("", include("solid_backend.contact.urls")),
    path("", include("solid_backend.content.urls")),
    path("", include("solid_backend.glossary.urls")),
    path("", include("solid_backend.message.urls")),
    path("", include("solid_backend.photograph.urls")),
    path("", include("solid_backend.quiz.urls")),
    path("", include("solid_backend.slideshow.urls")),
]

if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['solid_backend',
 'solid_backend.contact',
 'solid_backend.contact.migrations',
 'solid_backend.contact.tests',
 'solid_backend.contact.tests.conftest_files',
 'solid_backend.content',
 'solid_backend.content.migrations',
 'solid_backend.glossary',
 'solid_backend.glossary.migrations',
 'solid_backend.glossary.tests',
 'solid_backend.glossary.tests.conftest_files',
 'solid_backend.message',
 'solid_backend.message.migrations',
 'solid_backend.message.tests',
 'solid_backend.message.tests.conftest_files',
 'solid_backend.openzoom',
 'solid_backend.photograph',
 'solid_backend.photograph.migrations',
 'solid_backend.photograph.tests',
 'solid_backend.photograph.tests.conftest_files',
 'solid_backend.quiz',
 'solid_backend.quiz.migrations',
 'solid_backend.quiz.tests',
 'solid_backend.quiz.tests.conftest_files',
 'solid_backend.slideshow',
 'solid_backend.slideshow.migrations',
 'solid_backend.slideshow.tests',
 'solid_backend.slideshow.tests.conftest_files']

package_data = \
{'': ['*']}

install_requires = \
['django-cleanup==4.0.0',
 'django-mptt==0.11.0',
 'django-stdimage>=5.1.1,<6.0.0',
 'django>=3.0.7,<3.1.0',
 'djangorestframework==3.11.0',
 'mutagen>=1.44.0,<2.0.0',
 'pillow==7.1.2',
 'psycopg2-binary>=2.8.5,<3.0.0']

setup_kwargs = {
    'name': 'solid-backend',
    'version': '0.1.12',
    'description': 'Clean Django app for e-learning application with...',
    'long_description': '# S.O.L.I.D.-Backend\n\n<p align="center">\n<a href="https://codecov.io/gh/zentrumnawi/solid-backend">\n  <img src="https://codecov.io/gh/zentrumnawi/solid-backend/branch/master/graph/badge.svg" /> \n</a>\n<a href="https://travis-ci.com/zentrumnawi/solid-backend"> \n  <img src="https://travis-ci.com/zentrumnawi/solid-backend.svg?branch=master">\n</a>\n<a href="https://github.com/ambv/black">\n  <img alt="Code style: black" src="https://img.shields.io/badge/code%20style-black-000000.svg">\n</a>\n</p>\n\n## What is S.O.L.I.D.?\nThe project\'s name is an acronym for **S**ystematic **O**bject-**L**earning and **I**dentification, \nwhich is the purpose of this pluggable django app. It is here to provide a solid (hehe - :D) foundation for future eLearning apps.\nDuring the creation of two eLearning apps at Goethe University Frankfurt we figured out that a lot of disciplines require students to learn how to systematically analyse and classify certain objects. Those objects might be hand-samples of minerals, stones, stuffed animals, paintings or plants, mostly stored in archives with limited access - or none at all. To prevent us from repeating all steps in the creation of a back- and frontend, we focused on the question "Which components will all these apps have in common?" and this package is what we came up with.\nIn order to tackle the problem of limited access to hand-samples, we need to store and process high-definition images with the capability of a zoom feature to deliver HD images on any kind of device. Furthermore, we need a way to arrange the hand-samples in a structured way to provide the core feature. But that\'s not it. eLearning does not solely consist of content distribution. So we also thought of a glossary which the students have at hand without the need to refer to the lecture notes. Another module to display miscellaneous content is the slideshow feature which dombines text and image on separate pages.\nBeyond that, we implemented a quiz in order to give students the opportunity to evaluate their current level of understanding. To top it off, we figured it might be useful to display messages to communicate with our users. Messages can inform about updates, events in the context of a lecture, provide fun facts or make up an advent calender with daily tasks or puzzles. A contact form implementation provides a communication in the other direction.\n\nSo what does S.O.L.I.D provide in short:\n\n- A generic way to set up database models which can be structured in a hierarchical tree.\n- A simple way to store high-definition images which provides automatic creation of DZI-files for the usage of [OpenSeadragon](https://openseadragon.github.io/)\n- A Quiz-system with a variety of Question types.\n- A Glossary to provide subject-specific terminology.\n- A Message system which can be utilized in various ways.\n- A Slideshow system to provide content in a presentation style.\n- A Feedback form to facilitate bug reports and questions.\nSo if you are looking to build the backend of an eLearning app, you came to the right place.\nFor inspiration or just to see what it looks like to use `solid-backend` in the end have a look at [geomat-backend]()and/or [dive-backend]().\nIf you are also interested in the Frontend: We also have an Angular package which can be found [here]() and which is the foundation of the corresponding apps under geomat.uni-frankfurt.de and dive.uni-frankfurt.de\n\n\n## Get started\nRequirements for this package are:\n* Django >3.0.0\n* Djangorestframework 3.11.0\n* django-mptt 0.11.0\n* pillow 7.1.2\n* django-cleanup 4.0.0\n* psycopg2-binary 2.8.5\n\nPsycopg2 is important because we are using PostgreSQL specific databasefields. This means that\nyou are also required to use a PostgreSQL database.\n\n### Installation\n\nThe solid-backend package is distributed via PyPi so you can simply install it via\n\t\n\tpip install solid-backend\n\t\nAdd the apps to your `INSTALLED_APPS` for a bare minimum functionality\n\t\n\tsettings.py\n\t\n\tINSTALLED_APPS = [\n    ...,\n    "solid_backend.content",\n    "solid_backend.photograph",\n]\n\nor all apps\n\t\n\tsettings.py\n\t\n\tINSTALLED_APPS = [\n\t\t"solid_backend.contact",\n\t\t"solid_backend.content",\n\t\t"solid_backend.glossary",\n\t\t"solid_backend.message",\n\t\t"solid_backend.quiz",\n\t\t"solid_backend.slideshow",\n\t\t"solid_backend.photograph",\n\t]\n\nAfterwards, don\'t forget to add the url\'s to your url-config.\nHere, again, you can either decide which urls to include or include all of them:\n\nSpecific urls:\n\n\turls.py\n\t\n\turlpatterns = [\n\t\t...,\n\t\turl(r"", include("solid_backend.content.urls"), name="content"),\n\t\turl(r"", include("solid_backend.message.urls"), name="message"),\n\t\turl(r"", include("solid_backend.photograph.urls"), name="photograph"),\n\t\n\t]\n\nAll urls:\n\n\t\turls.py\n\t\n\turlpatterns = [\n\t\t...,\n\t\turl("", include("solid_backend.urls")),\n\t\t\n\t]\n\t\nAfter this you are ready to run the migrations and you are good to go.\n\n\n\n## Documentation\n\nDocumentation is available [here](https://app.gitbook.com/@zentrumnawi/s/dive/).\nThe Documentation is currently only available in german. If you are a non-german speaker\nand want to know more about something feel free to contact us directly via mail and we \nwill figure it out.\n\n## Coverage\n\nComing soon...\n\n## Try it out and local development\n\nFor a How-To guide see the README in the sample_project directory.\n\n',
    'author': 'Christian Grossmüller',
    'author_email': 'chgad.games@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/zentrumnawi/solid-backend',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import pytest

from solid_backend.photograph.models import Photograph
from solid_backend.quiz.models import QuizAnswer, QuizQuestion


@pytest.fixture
def quiz_question_model_class():
    return QuizQuestion


@pytest.fixture
def quiz_answer_model_class():
    return QuizAnswer


@pytest.fixture
def photograph_model_class():
    return Photograph



from django.contrib import admin
from django.contrib.contenttypes.admin import GenericTabularInline

from .forms import PhotographAdminForm, PhotographInlineFormSet
from .models import Photograph

fields = [
    "profile_position",
    ("img", "dzi_option"),
    "img_alt",
    "description",
    "dzi_file",
    "img_original_width",
    "img_original_height",
    ("length_value", "length_unit", "pixel_number"),
    "img_original_scale",
    "audio",
    "audio_duration",
    "date",
    "author",
    "license",
]
readonly_fields = [
    "dzi_file",
    "img_original_width",
    "img_original_height",
    "img_original_scale",
    "audio_duration",
]


class DeepZoomAdmin(admin.ModelAdmin):
    def delete_queryset(self, request, queryset):
        """
        Call DeepZoom model delete method for each object during bulk delete via admin
        action to ensure that the Deep Zoom directory trees will be deleted.
        """

        for object in queryset:
            object.delete()


class PhotographInline(GenericTabularInline):
    model = Photograph
    form = PhotographAdminForm
    formset = PhotographInlineFormSet
    extra = 1
    fields = fields
    readonly_fields = readonly_fields


class PhotographAdmin(DeepZoomAdmin):
    form = PhotographAdminForm
    fields = ["profile"] + fields
    readonly_fields = ["profile", "profile_position"] + readonly_fields
    list_display = ["id", "profile", "img", "dzi_option", "author", "license"]

    def profile(self, obj):
        if not obj.profile:
            return "-"
        return str(obj.profile)


admin.site.register(Photograph, PhotographAdmin)



from os import path

from django.db import migrations, models
from django.db.models import Q
from stdimage.models import JPEGFieldFile


def change_img_field(apps, schema_editor):
    """
    Replace image file with a relation to a photograph instance created from the image.
    """
    QuizQuestion = apps.get_model("quiz", "QuizQuestion")
    Photograph = apps.get_model("photograph", "Photograph")

    for obj in QuizQuestion.objects.exclude(Q(img_old="") | Q(img_old=None)):
        # Create Photograph instance with StdImage variations from image file
        filename = path.basename(obj.img_old.path)
        file = obj.img_old.open()

        instance = Photograph()
        instance.img.save(filename, file, save=False)
        variations = {
            "large": {
                "width": 1200,
                "height": None,
                "crop": False,
                "resample": 1,
                "kwargs": {},
                "name": "large",
            },
            "medium": {
                "width": 900,
                "height": None,
                "crop": False,
                "resample": 1,
                "kwargs": {},
                "name": "medium",
            },
            "small": {
                "width": 600,
                "height": None,
                "crop": False,
                "resample": 1,
                "kwargs": {},
                "name": "small",
            },
            "thumbnail": {
                "width": 100,
                "height": 100,
                "crop": True,
                "resample": 1,
                "kwargs": {},
                "name": "thumbnail",
            },
        }
        for _, variation in variations.items():
            JPEGFieldFile.render_variation(instance.img.name, variation)
        instance.img_alt = obj.img_alt
        instance.save()

        # Assign created Photograph instance to img field and delete image file
        obj.img.add(instance)
        obj.img_old.delete(save=True)


class Migration(migrations.Migration):

    dependencies = [
        ("photograph", "0005_implement_deep_zoom_fields"),
        ("quiz", "0003_add_verbose_name_to_text_fields"),
    ]

    operations = [
        migrations.RenameField(
            model_name="quizquestion", old_name="img", new_name="img_old",
        ),
        migrations.AddField(
            model_name="quizquestion",
            name="img",
            field=models.ManyToManyField(blank=True, to="photograph.Photograph"),
        ),
        migrations.RunPython(change_img_field, migrations.RunPython.noop),
        migrations.RemoveField(model_name="quizquestion", name="img_alt",),
        migrations.RemoveField(model_name="quizquestion", name="img_old",),
    ]
